// Pollyfill for app running on pre iOS 9 versions
if (!String.prototype.startsWith) {
    String.prototype.startsWith = function(searchString, position){
        position = position || 0;
        return this.substr(position, searchString.length) === searchString;
    };
}

var _appStartupVs=5;
var _usingAppStartupScript=true;
var _canUseSSL  = (!_isStaging && _base.indexOf("mobileinventor.com") !== -1) ? true : false;
var _baseDir    = _base; 
var _baseDirSSL = _baseDir.replace("http","https");
var _libraryPath = (_canUseSSL) ? _baseDirSSL +"applib/" : _baseDir +"applib/";
var _templatePath = (_canUseSSL) ? _baseDirSSL +"appdata/" : _baseDir +"appdata/";
var _versionCheckComplete=false, _windowLoaded, _appFilesThatHaveChanged;
var _cache, _syncSysCache, _sysData, _delayedSysData;

function getVersionInfo(){
    _sysData = localStorage.getItem("sysData_"+_appId);

    try {
        _sysData = JSON.parse(_sysData);
    } catch(e) {
        _sysData = null;
    }
    
    if (!_sysData){
        _sysData = {};
        _sysData.sysver=0;
        _sysData.appver=0;
        _sysData.resetSysVer=0;
        _sysData.fileInfo=null;
        _sysData.hasSysChanges=true;
        _sysData.hasAppChanges=true;
        _sysData.hasAppFileChanges=false;        
    }
    
}
function moveNewSysDataOver(sysData, newSysData){
    if (newSysData){
        var hasSysChanges = (!sysData.resetSysVer || newSysData.resetSysVer == 0 || newSysData.resetSysVer > sysData.resetSysVer);
        var hasAppChanges = (!sysData.appver || newSysData.appver == 0 || newSysData.appver > sysData.appver);
                
        _appFilesThatHaveChanged = getFilesThatHaveChanged(sysData.fileInfo, newSysData.fileInfo);
        
        sysData = JSON.parse(JSON.stringify(newSysData));
        sysData.hasSysChanges=hasSysChanges;
        sysData.hasAppChanges=hasAppChanges;
        sysData.hasAppFileChanges=(_appFilesThatHaveChanged) ? true : false;
        return sysData;
    }        
}

function getFilesThatHaveChanged(currInfo, newInfo){
    if (!newInfo) return null;
    if (!currInfo) return newInfo;

    var changes; 
    for (var property in newInfo) {
        if (newInfo.hasOwnProperty(property)) {
            if (!currInfo[property] || newInfo[property] > currInfo[property]) {
                if (!changes) changes = {};
                changes[property] = newInfo[property];
            }
        }
    }
    return changes;
}

function getSysData(triggerComplete, callback){   
    var dataStr = 'app_id=' + _appId + '&staging='+_isStaging; 
    $.ajax({
        type:"GET",
        async:true,
        url:_libraryPath + "getVersionInfo.php",
        data:dataStr,
        cache:false,
        crossDomain:true,
        success:function(data){
            try {
                data = JSON.parse(data);
            } catch(e) {
                data=null;
            }        

            _versionCheckComplete=true;
            if (triggerComplete) $(window).trigger("versionCheckComplete");
            if (callback) callback(data);
        },
        error:function(){
            console.log("Error getting versioning information!");
            if (triggerComplete) $(window).trigger("versionCheckComplete");
            if (callback) callback(null);
        }
    });
} 
function delayedVersionCheck(){
     _versionCheckComplete=false;
     
    // if pageloader not loaded then delay else if is app and not ready delay
    // We require pageloader as it has listener for any changes found via async check
    if ( (typeof _documentReady === "undefined" || !_documentReady)
    ||   (typeof _pageLoaded === "undefined" || !_pageLoaded)
    ||   (_isApp && (typeof _deviceReady === "undefined" || !_deviceReady)) )  {
        setTimeout(delayedVersionCheck, 800);
        return;
    }
	    
    getSysData(false, function(data){
        _delayedSysData = data;
        $(window).trigger("versionCheckComplete");
    }); 
}

var app = {
    // Application Constructor
    initialize: function() {
        this.bindEvents();
    },
    // Bind Event Listeners
    //
    // Bind any events that are required on startup. Common events are:
    // 'load', 'deviceready', 'offline', and 'online'.
    bindEvents: function() {
        document.addEventListener('deviceready', this.onDeviceReady, false);
        window.addEventListener("load", this.onWindowLoaded, false);
    },
    onWindowLoaded: function(e) {
        _windowLoaded=true;
    },
    // deviceready Event Handler
    //
    // The scope of 'this' is the event. In order to call the 'receivedEvent'
    // function, we must explicitly call 'app.receivedEvent(...);'
    onDeviceReady: function() {
        if (typeof device !== "undefined" && device.platform && device.platform.toLowerCase() === "android"){
            app.receivedEvent('deviceready');
        }else {
            // Something is causing the app to randomly freeze at startup.  Buest guess is the app cache.  Only true way to avoid it is putting in a small delay here
            setTimeout(function(){ app.receivedEvent('deviceready'); }, 1000);            
        }

        //app.receivedEvent('deviceready');
        //window.addEventListener('filePluginIsReady', function(){ app.receivedEvent('deviceready'); }, false);
    },
    // Update DOM on a Received Event
    receivedEvent: function(id) {
        //console.log('Received Event: ' + id);
        var me = this;
        if (id === "deviceready"){
            if (typeof device !== "undefined" && device.platform && device.platform.toLowerCase() === "android"){
                var vs = (device.version) ? device.version.substr(0,1) : null;
                if (vs && vs >= 6 && cordova.plugins.permissions){
                    var permissions = cordova.plugins.permissions;
                    permissions.hasPermission(permissions.WRITE_EXTERNAL_STORAGE,function(status){
                        if(!status.hasPermission) {
                            navigator.notification.confirm(
                                "This app needs access to your device's storage to store important system files that the app cannot operate without.  The Photos & Media permissions are only used if you want to send us a photo.", 	// message
                                function(button){
                                    if (button == 1){
                                        permissions.requestPermission(permissions.WRITE_EXTERNAL_STORAGE, 
                                            function(status){
                                                if (!status.hasPermission){
                                                    navigator.notification.alert("If you did not see a request for permission then please go to System->App Manager and change the app permissions there",
                                                        function () {
                                                            navigator.app.exitApp();
                                                        },"Permission Denied", "Exit App");
                                                }else {
                                                    me.versionChecks();                        
                                                }
                                            }, 
                                            function() {
                                                navigator.app.exitApp();
                                            });
                                    }else{
                                        navigator.app.exitApp();
                                    }
                                },      
                                'Storage Permission', 	// title
                                'Ok, Exit App'        	// buttonLabels					    
                            );
                        }else {
                            me.versionChecks();
                        }
                    },function(){
                        me.versionChecks();
                    });                        
                }else {
                    me.versionChecks();
                }
            }else {
                me.versionChecks();
            }
        }
    },
    versionChecks: function(){
        var me=this;
        
        localStorage.removeItem("syncedSysData_"+_appId);
        localStorage.removeItem("restartNeeded"+_appId);
        
        getVersionInfo();

        //Typical of new install
        if (!_sysData.appver || !_sysData.resetSysVer){
            getSysData(false, function(data){
                if (data) _sysData = moveNewSysDataOver(_sysData, data);
                me.setupCache(true);
            });            
        }else {
            me.setupCache(_sysData.hasSysChanges);
            delayedVersionCheck();            
        }
    },
    setupCache: function(resetCache){

        //First check to see if anything in the syncdata cache (from silent pushes or delayed version checks)
/*        window.requestFileSystem(LocalFileSystem.PERSISTENT, 0,
            function(fs){
                var directoryEntry = fs.root;
                directoryEntry.getDirectory("syncdata", {create: false, exclusive: false},
                    function(parent){
                        var directoryReader = parent.createReader();
                        directoryReader.readEntries(
                            function(entries){
                                if (entries.length > 0){
                                    for (var i = 0; i < entries.length; i++) {
                                        var fe = entries[i];
                                        fe.file(function (file) {
                                            console.log("found file in syncdata: " + file.name);
                                            
                                            var dirName = "";
                                            if (file.name === "style.css"){
                                                dirName = "data/appdata/css/";
                                            }else if (file.name === "jquery.fancybox.css" || file.name === "jquery.fancybox.pack.js"){
                                                dirName = "data/appdata/js/fancybox/";
                                            }else {
                                                dirName = "data/appdata/js/";
                                            }
                                            
                                            fs.root.getDirectory(dirName, {}, function(dirEntry) {
                                                debugger;
                                                fe.moveTo(dirEntry);
                                            }, function(error){
                                                console.log("error trying to move sync file to data file - "+error.code);
                                            });
                                        });
                                    }
                                }
                            },
                            function(error){
                                console.log("Unable to get list of files from syncdata directory: " + error.code);
                            });
                    },
                    // failed
                    function(error){
                        console.log("Unable to read syncdata directory: " + error.code);
                    });
            },
            //failed
            function(evt){
                console.log(evt.target.error.code);
            });
*/
        _syncSysCache = new CordovaFileCache({
             fs: new CordovaPromiseFS({ Promise: Promise }),
             mode: 'mirror', // or hash
             localRoot: 'syncdata',
             serverRoot: (_canUseSSL) ? _baseDirSSL :_baseDir,
             cacheBuster: true  
        });
        
        _syncSysCache.ready.then(function(list){
            if (resetCache){
                _syncSysCache.clear().then(
                    function(){
                        app.setupCacheFinish(resetCache);
                    },
                    function(){
                        console.log("problem clearing sync cache!!");
                        app.setupCacheFinish(resetCache);
                    }
                );
                
            }else {
                var movedCnt=0;
                for (var i=0; i<list.length; i++){
                    var fs = new CordovaPromiseFS({ Promise: Promise });
                    var fn = list[i];
                    var dest = fn.replace("syncdata","data");
                    dest = dest.substring(0,(dest.lastIndexOf('/') + 1));
                    fs.move(fn,dest).then(function(fe){
                        console.log("moved "+fn+" from syncdata to data");
                        ++movedCnt;
                        if (movedCnt === list.length) app.setupCacheFinish(resetCache);
                    }, function(err){
                        console.log("error moving "+fn+" from syncdata to data:"+err);
                        ++movedCnt;
                        if (movedCnt === list.length) app.setupCacheFinish(resetCache);
                    });
                }
                if (list.length === 0) app.setupCacheFinish(resetCache);
            }
        });
                     
    },
    setupCacheFinish: function(resetCache){
        _cache = new CordovaFileCache({
             fs: new CordovaPromiseFS({ // An instance of CordovaPromiseFS is REQUIRED
                 Promise: Promise // <-- your favorite Promise lib (REQUIRED)
             }),
             mode: 'mirror', // or hash
             localRoot: 'data', 
             serverRoot: (_canUseSSL) ? _baseDirSSL :_baseDir, 
             cacheBuster: true  
         });
        
        _cache.ready.then(function(list){
            if (resetCache) {
                console.log("clearing main system cache!");
                _cache.clear().then(
                    function(){
                        app.loadCache(_cache, true)
                    },
                    function(){
                        console.log("problem clearing cache!!");
                    });
            }else {
                console.log("main system cache doesn't need to be cleared");
                app.loadCache(_cache, false);
            }
         });         
    },
    writeScriptOrCSS: function(type, path, onload){
        var c,s;
        if (type === "css"){
            c = document.createElement("link");
            c.href=path;
            c.rel="stylesheet";
            c.type="text/css";
            c.onload=onload;
            document.getElementsByTagName("head")[0].appendChild(c);
        }else {
            s = document.createElement( 'script' );
            s.setAttribute( 'src', path);
            s.onload=onload;
            document.body.appendChild(s);            
        }
    },
    loadCache: function(cache, cacheReset){
        console.log("loading main system cache");
            
        var changes;
        // if cache was not cleared then it means that we are doing version check in async so we rely on any previously notified changesin storage, 
        // remove them from the cache and add the new info about them in _sysData.fileInfo
        if (!cacheReset && _sysData.fileInfo){
            changes = _appFilesThatHaveChanged;
            if (changes){
                console.log("...found files to update in main system cache");
                for (var property in changes) {
                    if (changes.hasOwnProperty(property)) {
                        _sysData.fileInfo[property] = changes[property];
                        if (property === "style.css"){
                            cache.remove(_templatePath + "css/style.css");
                        }else if (property === "jquery.fancybox.css" || property === "jquery.fancybox.pack.js"){
                            cache.remove(_templatePath + "js/fancybox/"+property);
                        }else {
                            cache.remove(_templatePath + "js/"+property);
                        }
                    }
                }                
            }
        }
        
        cache.add(_templatePath + 'css/style.css');
        cache.add(_templatePath + 'js/oneTime-min.js');
        cache.add(_templatePath + 'js/iscroll.js');
        cache.add(_templatePath + 'js/common.js');
        cache.add(_templatePath + 'js/pageLoader-v2.js');
        cache.add(_templatePath + 'js/fancybox/jquery.fancybox.css');
        cache.add(_templatePath + 'js/fancybox/jquery.fancybox.pack.js');
        
        var onProgress = function(e){
            /*var progress ="Progress: " 
                + e.queueIndex // current download index 
                + " " 
                + e.queueSize; // total files to download
            console.log("Download Progress = "+progress); */            
        }
        // Note that we append timestamp to all of the file:// paths as iOS caches those paths even when we do a location.reload(true).
        // The timestamp will cause it to actually go to filesystem to get file.
        cache.download(onProgress, false).then(
            // Download complete (if any)
            function(cache){
                var c,s;
                // Walk through css file and remove relative paths replacing them with absolute path.
                // If we find an absolute already then we have altered it before.  In that case, we
                // check cache and replace it with cached path if available.
                var fs = new CordovaPromiseFS({ Promise: Promise });
                var fn = cache.toPath(_templatePath + 'css/style.css');
                var timestamp = _sysData.fileInfo["style.css"];
                fs.read(fn).then(function(data){
                    var newData = data.replace(/url\(.*?\)/ig, function(match){
                        var path;
                        if (match.startsWith('url(\"./')){
                            path = _templatePath + "css/" + match.replace('url(\"./', '').replace('")', '');
                        }else if (match.startsWith('url(\"../')){
                            path = _templatePath + match.replace('url(\"../', '').replace('")', '');
                        }else if (match.startsWith('url("'+_templatePath)){
                            path = match.replace('url("', '').replace('")', '');
                        }else if (match.indexOf("file://") !== -1 || match.indexOf("cdvfile://") !== -1){
                            // actual path of cached files can change when app is restarted so we replace it.
                            path = match.replace('")', '').replace('\")', '');
                            path = path.substr(path.lastIndexOf("/data/") + 6);
                            path += '?'+timestamp;
                        }
                        
                        if (path){
                            path = cache.get(path);
                            if (path.startsWith(_templatePath)) cache.add(path);
                            return 'url("'+path+'?'+timestamp+'")';                            
                        }else {
                            return match;
                        }
                    });
                    
                    if (cache.isDirty()) cache.download(function(){},false).then(function(cache){}, function(failedDownloads){});
                    
                    cache._fs.write(fn, newData).then(function(e){
                        
                        app.writeScriptOrCSS("css", cache.get(_templatePath + 'css/style.css?')+'?'+timestamp, function(){
                            //console.log("style.css written");
                            app.writeScriptOrCSS("script", cache.get(_templatePath + 'js/oneTime-min.js')+'?'+_sysData.fileInfo["oneTime-min.js"], function(){
                                //console.log("oneTime-min written");
                                app.writeScriptOrCSS("script", cache.get(_templatePath + 'js/iscroll.js')+'?'+_sysData.fileInfo["iscroll.js"], function(){
                                    //console.log("iscroll written");
                                    app.writeScriptOrCSS("script", cache.get(_templatePath + 'js/common.js')+'?'+_sysData.fileInfo["common.js"], function(){
                                        //console.log("common written");
                                        app.writeScriptOrCSS("css", cache.get(_templatePath + 'js/fancybox/jquery.fancybox.css')+'?'+_sysData.fileInfo["jquery.fancybox.css"], function(){
                                            //console.log("fancy css written");
                                            app.writeScriptOrCSS("script", cache.get(_templatePath + 'js/fancybox/jquery.fancybox.pack.js')+'?'+_sysData.fileInfo["jquery.fancybox.pack.js"], function(){
                                                //console.log("fancy written");
                                                app.writeScriptOrCSS("script", cache.get(_templatePath + 'js/pageLoader-v2.js')+'?'+_sysData.fileInfo["pageLoader-v2.js"], function(){
                                                    //console.log("pageloader-v2 written");
                                                    console.log("...main system cache load complete");
                                                });
                                            });
                                        });                                    
                                    });
                                });
                            });
                        });
                    }, function(err){
                        console.log("could not write altered css file to filesystem - "+err);
                    });

                }, function(err){
                    console.log("error reading style.css - "+err);
                });
               
            },function(failedDownloads) {
                console.log("download error - "+failedDownloads);
            }
        );
    }
};

$(document).ready(function() {
    app.initialize();
});
